// component-from-stream
/**
 * @license
 * Copyright 2018 Stephane M. Catala
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * Limitations under the License.
 */

const createSubject = require('rx-subject').default
export default function createComponentFromStreamFactory(
  ComponentCtor,
  fromES,
  toESOrOpts = identity,
  opts = {}
) {
  if (!isFunction(toESObservableOrOpts)) {
    return createComponentFromStreamFactory(
      ComponentCtor,
      fromES,
      void 0,
      toESOrOpts
    )
  }

  const toES = toESOrOpts

  return function createComponentFromStream(
    render,
    operator = identity,
    onprops = identity,
    ...middlewares
  ) {
    return class ComponentFromStreamClass extends ComponentCtor {
      constructor() {
        super(...arguments)
        this.state = {}
        this._setProps = props => this.setState({ props })
        this._applyMiddleware = (dispatch, middleware) =>
          middleware(dispatch, this._input$, fromES, toES)
        this._unsubscribe = () => {
          this._sub.unsubscribe()
        }
      }

      render() {
        return !this.state.props ? null : render(this.state.props)
      }

      componentWillMount() {
        // debugger
        this._inputs = createSubject()
        this._input$ = fromES(this._inputs.source$)
        this._sub = toES(operator(this._input$)).subscribe(
          this._setProps
        )
        this._inputs.source$.subscribe(
          nop,
          this._unsubscribe,
          this._unsubscribe
        )
        const dispatch = middlewares.reduceRight(
          this._applyMiddleware,
          this._inputs.sink.next
        )
        this._onProps = onprops(dispatch)
        this._onProps(this.props)
      }

      componentWillReceiveProps(props) {
        this._onProps(props)
      }

      componentWillUnmount() {
        this._inputs.sink.complete()
      }

      shouldComponentUpdate(_, state) {
        return state.props !== this.state.props
      }
    }
  }
}

function isFunction(v) {
  return typeof v === 'function'
}

function identity(val) {
  return val
}

function nop() {}

// redux-middleware
/**
 * @license
 * Copyright 2018 Stephane M. Catala
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * Limitations under the License.
 */

const Subject = require('rx-subject').default

export default function(reducer, ...effects) {
  return function(dispatch, state$, fromES, toES) {
    let state
    state$.subscribe(
      function(s) {
        state = s
      },
      end,
      end
    )
    const events = Subject()
    const event$ = fromES(events.source$)
    const subs = effects.reduce(
      function(subs, effect) {
        const effect$ = toES(effect(event$, state$))
        return subs.concat(effect$.subscribe(events.sink.next))
      },
      [events.source$.subscribe(reduce)]
    )

    return events.sink.next

    function reduce(event) {
      dispatch(reducer(state, event))
    }

    function end() {
      events.sink.complete()
      while (subs.length) {
        subs.pop().unsubscribe()
      }
    }
  }
}
