/**
 * @license
 * Copyright 2018 Stephane M. Catala
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * Limitations under the License.
 */

const Subject = require('rx-subject').default

export default function(reducer, ...effects) {
	return function(dispatch, state$, fromES, toES) {
		let state
		state$.subscribe(
			function(s) {
				state = s
			},
			end,
			end
		)
		const events = Subject()
		const event$ = fromES(events.source$)
		const subs = effects.reduce(
			function(subs, effect) {
				const effect$ = toES(effect(event$, state$))
				return subs.concat(effect$.subscribe(events.sink.next))
			},
			[events.source$.subscribe(reduce)]
		)

		return events.sink.next

		function reduce(event) {
			dispatch(reducer(state, event))
		}

		function end() {
			events.sink.complete()
			while (subs.length) {
				subs.pop().unsubscribe()
			}
		}
	}
}
